==> peekabot-0.6.0.tar.bz2: peekabot-0.6.0/src/gui/Viewport.cc <==
/*
 * Copyright Anders Boberg 2006-2007.
 * Copyright Staffan Gimåker 2006-2007, 2009.
 *
 * ---
 *
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <FL/Fl.H>
#include <FL/fl_draw.H>
#include <boost/shared_ptr.hpp>
#include <boost/filesystem/path.hpp>
#include <boost/filesystem/operations.hpp>

#include "Viewport.hh"
#include "ViewFrame.hh"
#include "../CameraObject.hh"
#include "../Types.hh"
#include "../Singleton.hh"
#include "../renderer/Renderer.hh"
#include "../IDTranslationTable.hh"
#include "UserInterface.hh"
#include "UserInterfaceUtilities.hh"
#include "../SceneTree.hh"
#include "../SceneTreePtr.hh"
#include "../SnapshotWriter.hh"
#include "../Configuration.hh"
#include "Manipulator.hh"
#include "../FsToolbox.hh"

#include <boost/format.hpp>

using namespace peekabot;
using namespace gui;
using namespace renderer;
using namespace boost::posix_time;

Viewport::Viewport(int x, int y, int w, int h, char* l) :
    Fl_Gl_Window(x, y, w, h, l),
    m_last_action_time(microsec_clock::local_time()),
    m_gl_context_is_setup(false),
    m_is_fullscreen(false),
    m_save_snapshot(false),
    m_continual_snapshots_enabled(false)
{    
    end();
    
    mode(FL_DOUBLE | FL_RGB | FL_DEPTH  | FL_STENCIL | FL_MULTISAMPLE);

    m_camera = 0;

    m_visible_layers[0] = true;
    for(int i = 1; i < 16; ++i)
        m_visible_layers[i] = false;
}

Viewport::Viewport(Viewport *vp, int x, int y, int w, int h)
    : Fl_Gl_Window(x, y, w, h, "peekabot view"),
      m_camera(vp->m_camera),
      m_last_action_time(microsec_clock::local_time()),
      m_gl_context_is_setup(false),
      m_is_fullscreen(true)
{
    end();
    fullscreen();
    clear_border();
    mode(FL_DOUBLE | FL_RGB | FL_DEPTH  | FL_STENCIL | FL_MULTISAMPLE);
    
    
    for(int i = 0; i < 16; ++i)
        m_visible_layers[i] = vp->m_visible_layers[i];
    show();
    set_modal();
}

Viewport::~Viewport()
{
}

int Viewport::handle(int msg)
{      
    // Calculate delta value for mouse pointer...
    int x = Fl::event_x();
    int y = Fl::event_y();
    int dx = x - m_old_x;
    int dy = y - m_old_y;

    int button = Fl::event_button();
    int state = Fl::event_state();

    // Don't send actions too often...
    ptime current_time = microsec_clock::local_time();

    switch(msg)
    {
        case FL_ENTER:
            return 1;
        case FL_PUSH:         
            m_has_dragged = false;
            m_old_x = Fl::event_x();
            m_old_y = Fl::event_y();
            return 1;
        case FL_RELEASE:
            if(m_has_dragged && (dx > 0 || dy > 0))
            {
                TheUIUtilities::instance().get_manipulator()->
                    viewport_drag(this, dx, dy, button, state);
            }
            else if(!m_has_dragged)
                TheUIUtilities::instance().get_manipulator()->
                    viewport_click(this, x, y, button, state);
            return 1;
        case FL_DRAG:
            // Check to see that at least one action interval has passed
            // since our last visit here, else bail out.
            m_has_dragged = true;
            if(current_time < m_last_action_time +
               TheUserInterface::instance().get_action_interval())
                return 1;

            m_last_action_time = current_time;
            m_old_x = x;
            m_old_y = y;
            TheUIUtilities::instance().get_manipulator()->
                viewport_drag(this, dx, dy, button, state);
            return 1;
        case FL_MOUSEWHEEL:            
            TheUIUtilities::instance().get_manipulator()->
                viewport_scroll(this, Fl::event_dy(), state);
            return 1;
        case FL_MOVE:
            TheUIUtilities::instance().get_manipulator()->
            viewport_move(this, dx, dy, state);
            return 1;
        default:
            return Fl_Gl_Window::handle(msg);
    }
}

void Viewport::draw()
{
    {
        // We need to make sure that the camera of this viewport is still
        // in the scene.
        SceneTreePtr ptr = TheSceneTree::acquire();
        if(!get_camera(ptr))
           return;
    }

    boost::shared_ptr<SnapshotWriter> collector;

    if( m_save_snapshot )
    {
        collector = boost::shared_ptr<SnapshotWriter>(
            new SnapshotWriter(
                generate_snapshot_filename("snapshot_", ".png")));
        m_save_snapshot = false;
    }
    else if( m_continual_snapshots_enabled )
    {
        collector = boost::shared_ptr<SnapshotWriter>(
            new SnapshotWriter(
                generate_snapshot_filename(m_sequence_prefix, ".png")));
    }
    
    if(TheUserInterface::instance().get_gl_context() != 0)
    {
        if(!m_gl_context_is_setup)
       {
            context(TheUserInterface::instance().get_gl_context());
            make_current();
            m_gl_context_is_setup = true;
        }

        if(!valid())
        {
            context(TheUserInterface::instance().get_gl_context());
            //make_current();
        }
        TheRenderer::instance().render(
            w(), h(), 
            TheIDTranslationTable::instance().get_primitive_id(m_camera), 
            m_visible_layers,
            collector.get());
    }
}


CameraObject *Viewport::get_camera(SceneTreePtr &ptr)
{
    // We need to make sure that the camera of this viewport is still
    // in the scene.
    CameraObject *camera = dynamic_cast<CameraObject *>(
        ptr->get_object(m_camera));

    if(!camera)
        ((ViewFrame*)parent())->remove_camera(m_camera);

    return camera;
}

void Viewport::set_camera(ObjectID camera)
{
    m_camera = camera;
}

void Viewport::save_snapshot() throw()
{
    m_save_snapshot = true;
    redraw();
}

void Viewport::toggle_continual_snapshots(bool enable) throw()
{
    if( enable != m_continual_snapshots_enabled )
    {
        m_continual_snapshots_enabled = enable;

        if( enable )
        {
            fs::path snapshot_dir(
                TheConfiguration::instance().get_option<fs::path>(
                    "resources.snapshot_path", 
                    fs::get_resource_path() / fs::make_path("snapshots")));

            fs::path seq_path;
            static unsigned int n_seq = 1;

            do
            {
                char buf[256];
                sprintf(buf, "sequence_%02u", n_seq++);
                seq_path = fs::make_path(buf);
            }
            while( boost::filesystem::exists(snapshot_dir / seq_path) );

            fs::create_dir_if_inexistant(snapshot_dir / seq_path);
            m_sequence_prefix = (seq_path / fs::make_path("frame_")).string();
        }
    }
}

std::string Viewport::generate_snapshot_filename(
    const std::string &prefix, const std::string &postfix) const throw()
{
    fs::path snapshot_dir(
        TheConfiguration::instance().get_option<fs::path>(
            "resources.snapshot_path", 
            fs::get_resource_path() / fs::make_path("snapshots")));

    static std::map<std::string, unsigned int> counter_map;
    char buf[256];
    fs::path snapshot_path;
    do
    {
        sprintf(buf, "%s%04u%s", 
                prefix.c_str(), 
                ++counter_map[prefix], 
                postfix.c_str());
        snapshot_path = snapshot_dir / fs::make_path(buf);
    }
    while( boost::filesystem::exists(snapshot_path) );

    return snapshot_path.string();
}

==> peekabot-0.7.0.tar.bz2: peekabot-0.7.0/src/gui/Viewport.cc <==
/*
 * Copyright Anders Boberg 2006-2007.
 * Copyright Staffan Gimåker 2006-2007, 2009.
 *
 * ---
 *
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <FL/Fl.H>
#include <FL/fl_draw.H>
#include <boost/shared_ptr.hpp>
#include <boost/filesystem/path.hpp>
#include <boost/filesystem/operations.hpp>

#include "Viewport.hh"
#include "ViewFrame.hh"
#include "../CameraObject.hh"
#include "../Types.hh"
#include "../Singleton.hh"
#include "../RendererRelay.hh"
#include "UserInterface.hh"
#include "UserInterfaceUtilities.hh"
#include "../SceneTree.hh"
#include "../SceneTreePtr.hh"
#include "../SnapshotWriter.hh"
#include "../Configuration.hh"
#include "Manipulator.hh"
#include "../FsToolbox.hh"

#include <boost/format.hpp>

using namespace peekabot;
using namespace gui;
using namespace boost::posix_time;

Viewport::Viewport(int x, int y, int w, int h, char* l) :
    Fl_Gl_Window(x, y, w, h, l),
    m_last_action_time(microsec_clock::local_time()),
    m_gl_context_is_setup(false),
    m_is_fullscreen(false),
    m_save_snapshot(false),
    m_continual_snapshots_enabled(false)
{    
    end();
    
    mode(FL_DOUBLE | FL_RGB | FL_DEPTH  | FL_STENCIL | FL_MULTISAMPLE);

    m_camera = 0;

    m_visible_layers[0] = true;
    for(int i = 1; i < 16; ++i)
        m_visible_layers[i] = false;
}

Viewport::Viewport(Viewport *vp, int x, int y, int w, int h)
    : Fl_Gl_Window(x, y, w, h, "peekabot view"),
      m_camera(vp->m_camera),
      m_last_action_time(microsec_clock::local_time()),
      m_gl_context_is_setup(false),
      m_is_fullscreen(true)
{
    end();
    fullscreen();
    clear_border();
    mode(FL_DOUBLE | FL_RGB | FL_DEPTH  | FL_STENCIL | FL_MULTISAMPLE);
    
    
    for(int i = 0; i < 16; ++i)
        m_visible_layers[i] = vp->m_visible_layers[i];
    show();
    set_modal();
}

Viewport::~Viewport()
{
}

int Viewport::handle(int msg)
{      
    // Calculate delta value for mouse pointer...
    int x = Fl::event_x();
    int y = Fl::event_y();
    int dx = x - m_old_x;
    int dy = y - m_old_y;

    int button = Fl::event_button();
    int state = Fl::event_state();

    // Don't send actions too often...
    ptime current_time = microsec_clock::local_time();

    switch(msg)
    {
        case FL_ENTER:
            return 1;
        case FL_PUSH:         
            m_has_dragged = false;
            m_old_x = Fl::event_x();
            m_old_y = Fl::event_y();
            return 1;
        case FL_RELEASE:
            if(m_has_dragged && (dx > 0 || dy > 0))
            {
                TheUIUtilities::instance().get_manipulator()->
                    viewport_drag(this, dx, dy, button, state);
            }
            else if(!m_has_dragged)
                TheUIUtilities::instance().get_manipulator()->
                    viewport_click(this, x, y, button, state);
            return 1;
        case FL_DRAG:
            // Check to see that at least one action interval has passed
            // since our last visit here, else bail out.
            m_has_dragged = true;
            if(current_time < m_last_action_time +
               TheUserInterface::instance().get_action_interval())
                return 1;

            m_last_action_time = current_time;
            m_old_x = x;
            m_old_y = y;
            TheUIUtilities::instance().get_manipulator()->
                viewport_drag(this, dx, dy, button, state);
            return 1;
        case FL_MOUSEWHEEL:            
            TheUIUtilities::instance().get_manipulator()->
                viewport_scroll(this, Fl::event_dy(), state);
            return 1;
        case FL_MOVE:
            TheUIUtilities::instance().get_manipulator()->
            viewport_move(this, dx, dy, state);
            return 1;
        default:
            return Fl_Gl_Window::handle(msg);
    }
}

void Viewport::draw()
{
    {
        // We need to make sure that the camera of this viewport is still
        // in the scene.
        SceneTreePtr ptr = TheSceneTree::acquire();
        if(!get_camera(ptr))
           return;
    }

    boost::shared_ptr<SnapshotWriter> collector;

    if( m_save_snapshot )
    {
        collector = boost::shared_ptr<SnapshotWriter>(
            new SnapshotWriter(
                generate_snapshot_filename("snapshot_", ".png")));
        m_save_snapshot = false;
    }
    else if( m_continual_snapshots_enabled )
    {
        collector = boost::shared_ptr<SnapshotWriter>(
            new SnapshotWriter(
                generate_snapshot_filename(m_sequence_prefix, ".png")));
    }
    
    if(TheUserInterface::instance().get_gl_context() != 0)
    {
        if(!m_gl_context_is_setup)
       {
            context(TheUserInterface::instance().get_gl_context());
            make_current();
            m_gl_context_is_setup = true;
        }

        if(!valid())
        {
            context(TheUserInterface::instance().get_gl_context());
            //make_current();
        }
        TheRendererRelay::instance().render(
            w(), h(), 
            m_camera,
            m_visible_layers,
            collector.get());
    }
}


CameraObject *Viewport::get_camera(SceneTreePtr &ptr)
{
    // We need to make sure that the camera of this viewport is still
    // in the scene.
    CameraObject *camera = dynamic_cast<CameraObject *>(
        ptr->get_object(m_camera));

    if(!camera)
        ((ViewFrame*)parent())->remove_camera(m_camera);

    return camera;
}

void Viewport::set_camera(ObjectID camera)
{
    m_camera = camera;
}

void Viewport::save_snapshot() throw()
{
    m_save_snapshot = true;
    redraw();
}

void Viewport::toggle_continual_snapshots(bool enable) throw()
{
    if( enable != m_continual_snapshots_enabled )
    {
        m_continual_snapshots_enabled = enable;

        if( enable )
        {
            fs::path snapshot_dir(
                TheConfiguration::instance().get_option<fs::path>(
                    "resources.snapshot_path", 
                    fs::get_resource_path() / fs::make_path("snapshots")));

            fs::path seq_path;
            static unsigned int n_seq = 1;

            do
            {
                char buf[256];
                sprintf(buf, "sequence_%02u", n_seq++);
                seq_path = fs::make_path(buf);
            }
            while( boost::filesystem::exists(snapshot_dir / seq_path) );

            fs::create_dir_if_inexistant(snapshot_dir / seq_path);
            m_sequence_prefix = (seq_path / fs::make_path("frame_")).string();
        }
    }
}

std::string Viewport::generate_snapshot_filename(
    const std::string &prefix, const std::string &postfix) const throw()
{
    fs::path snapshot_dir(
        TheConfiguration::instance().get_option<fs::path>(
            "resources.snapshot_path", 
            fs::get_resource_path() / fs::make_path("snapshots")));

    static std::map<std::string, unsigned int> counter_map;
    char buf[256];
    fs::path snapshot_path;
    do
    {
        sprintf(buf, "%s%04u%s", 
                prefix.c_str(), 
                ++counter_map[prefix], 
                postfix.c_str());
        snapshot_path = snapshot_dir / fs::make_path(buf);
    }
    while( boost::filesystem::exists(snapshot_path) );

    return snapshot_path.string();
}
